.include  "m48def.inc"    ;definicje dla ATmega48
;.include  "m88def.inc"    ;definicje dla ATmega88
;.include  "m168def.inc"   ;definicje dla ATmega168
;.include  "m328def.inc"   ;definicje dla ATmega328

.cseg             ;pami programu (FLASH)
.org      $0000   ;wektory przerwa (dla ATmega48/88: 26 wektorw 2-bajtowych, o adresach $0000-$0019,
                  ;dla ATmega168/328: 26 wektorw 4-bajtowych, o adresach $0000-$0032).

;Jeli adne przerwania nie bd uywane, to w tym miejscu mona bezporednio umieci kod programu

;Wektor nr 1 (Reset)
          ;rjmp    RESET   ;dla ATmega48/88 musi to by instrukcja RJMP o rozmiarze 2 bajtw,
          ;jmp     RESET   ;dla ATmega168/328 musi to by instrukcja JMP o rozmiarze 4 bajtw,
                          ;poniewa wszystkie wektory w tych mikrokontrolerach maj taki rozmiar.

;RESET:
;Inicjalizacja wskanika stosu (zbdna dla ATmega48/88/168/328)
          ;ldi     R16,LOW(RAMEND)
          ;out     SPL,R16
          ;ldi     R16,HIGH(RAMEND)
          ;out     SPH,R16
          
;Konfiguracja mikrokontrolera
          ldi     R16,$01   ;czstotliwo pracy mikrokontrolera (CLK): 1-20 [MHz]
          mov     R15,R16
          ldi     R16,$EE   ;zatrzymanie zbdnych moduw (TWI, SPI, USART, TC0, TC1, TC2)
          sts     $64,R16   ;zapis rejestru PRR
          ldi     R16,$80   ;wyczenie zasilania komparatora analogowego
          sts     $50,R16   ;zapis rejestru ACSR
          ldi     R16,$FF
          out     PORTB,R16 ;ustawienie pinw PB0-PB7 jako wej z wysokim stanem (pull-up)
          out     DDRD,R16  ;ustawienie pinw PD0-PD7 jako wyj z niskim stanem
          ldi     R16,$30
          out     PORTD,R16 ;ustawienie wysokiego stanu na wyjciach PD4-PD5 (PD0-PD3/PD6-PD7 stan niski)
          ldi     R16,$3E
          out     PORTC,R16 ;ustawienie pinw PC1/ADC1-PC5/ADC5 jako wej z wysokim stanem (pull-up)
          ldi     R16,$01   ;wyczenie cyfrowego bufora wejciowego na pinie PC0/ADC0
          sts     $7E,R16   ;zapis rejestru DIDR0

;Konfiguracja wywietlacza LCD 2x16 znakw (HD44780)
          rcall   LCDi      ;inicjalizacja LCD (wczenie trybu 4-bitowego)
          ldi     R18,$28   ;ustawienie opcji: 2 linie, znaki 5x8 pikseli, interfejs 4-bit
          rcall   LCDwi     ;zapis instrukcji do LCD
          ldi     R18,$0C   ;wczenie LCD bez kursora
          rcall   LCDwi     ;zapis instrukcji do LCD
          rcall   LCDclr    ;czyszczenie LCD
          ldi     ZL,LOW(Text0<<1)
          ldi     ZH,HIGH(Text0<<1)
          rcall   LCDwf     ;wywietlenie na LCD cigu ASCIIZ z pamici FLASH
          ldi     R18,$C0   ;ustawienie kursora na pozycji 1 w 2 linii
          rcall   LCDwi     ;zapis instrukcji do LCD
          rcall   LCDwf     ;wywietlenie na LCD cigu ASCIIZ z pamici FLASH

;Gwna ptla programu
Loop:
;Odczyt stanu zworki J1 (pin PC5/ADC5), sucej do wyboru wejcia dla przetwornika ADC
          in      R16,PINC  ;odczyt stanu pinw PC0/ADC0-PC6
          andi    R16,$20
          brne    Loop0     ;skok, jeli na pinie PC5/ADC5 jest stan wysoki; zworka J1=OFF (rozwarta)
;Zworka J1=ON (zwarta): wybranie wewntrznego rda napicia odniesienia (1.1V) jako wejcia dla ADC
          ldi     R16,$4E   ;wybranie napicia odniesienia z pinu AVCC i wejcia dla ADC
          sts     $7C,R16   ;zapis rejestru ADMUX
          rjmp    Loop1
;Zworka J1=OFF (rozwarta): wybranie pinu PC0/ADC0 jako wejcia dla ADC
Loop0:    ldi     R16,$40   ;wybranie napicia odniesienia z pinu AVCC i wejcia dla ADC
          sts     $7C,R16   ;zapis rejestru ADMUX
;Uruchomienie konwersji ADC
Loop1:    ldi     R16,$C3   ;wczenie ADC i wykonanie pojedynczej konwersji z f=CLK/8 (125 kHz)
          sts     $7A,R16   ;zapis rejestru ADCSRA
;Czekanie na zakoczenie konwersji przez ADC
Loop2:    lds     R16,$7A   ;odczyt rejestru ADCSRA
          andi    R16,$40
          brne    Loop2     ;skok, jeli bit ADSC=1; konwersja trwa
;Odczyt wyniku konwersji ADC
          lds     R23,$78   ;odczyt rejestru ADCL (modszy bajt wyniku)
          lds     R16,$79   ;odczyt rejestru ADCH (starszy bajt wyniku)
;Konwersja 16-bitw wyniku do systemu HEX-ASCIIZ: [R16:R23] => "NNNN"
          ldi     ZL,LOW(sbuf)
          ldi     ZH,HIGH(sbuf) ;adres bufora wyjciowego w pamici SRAM
          rcall   ConBH     ;konwersja bajtu do systemu HEX-ASCIIZ
          mov     R16,R23   ;modszy bajt wyniku
          rcall   ConBH     ;konwersja bajtu do systemu HEX-ASCIIZ
;Wywietlenie szesnastkowej wartoci wyniku na LCD
          sbiw    ZL,$04
          ldi     R18,$C7   ;ustawienie kursora na pozycji 8 w 2 linii
          rcall   LCDwi     ;zapis instrukcji do LCD
          rcall   LCDws     ;wywietlenie na LCD cigu ASCIIZ z pamici SRAM
          ldi     R16,$FF   ;opnienie 25.5 ms
          rcall   Wait      ;ptla opniajca
          rcall   Wait      ;ptla opniajca
          rcall   Wait      ;ptla opniajca
          rcall   Wait      ;ptla opniajca
          rjmp    Loop

;Zapis szesnastkowej wartoci bajtu do pamici SRAM w postaci cigu ASCIIZ: "NN"
ConBH:    ldi     R17,$02
ConBH0:   swap    R16     ;bajt do konwersji
          mov     R18,R16
          andi    R18,$0F
          subi    R18,$D0
          cpi     R18,$3A
          brlt    ConBH1
          subi    R18,$F9 ;subi R18,$D9 - mae litery (a-f)
ConBH1:   st      Z+,R18
          dec     R17
          brne    ConBH0
          st      Z,R17   ;wyjcie danych
          ret

;Inicjalizacja wywietlacza LCD
LCDi:
;Czekanie na ustabilizowanie si napicia zasilania
          ldi     R16,$C8   ;opnienie 20 ms
          rcall   Wait      ;ptla opniajca
          rcall   Wait      ;ptla opniajca
;Wysanie trzech instrukcji $30 do LCD
          ldi     R18,$03
LCDi0:    ldi     R16,$B3
          out     PORTD,R16 ;E=1, RS=0 (PD4-PD5=1)
          ;(wait 4 CL)
          cbi     PORTD,$07 ;E=0, RS=0 (PD4-PD5=1)
          ldi     R16,$32   ;opnienie 5 ms
          rcall   Wait      ;ptla opniajca
          dec     R18
          brne    LCDi0
;Wysanie instrukcji $20 do LCD (wczenie trybu 4-bitowego)
          ldi     R16,$B2
          out     PORTD,R16 ;E=1, RS=0 (PD4-PD5=1)
          ;(wait 4 CL)
          cbi     PORTD,$07 ;E=0, RS=0 (PD4-PD5=1)
          ldi     R16,$0A   ;opnienie 1 ms
          rcall   Wait      ;ptla opniajca
          ret

;Zapis instrukcji do wywietlacza LCD (tryb 4-bitowy)
LCDwi:    ldi     R20,$B0   ;E=1, RS=0 (PD4-PD5=1)
          rjmp    LCDwc0
;Zapis znaku do wywietlacza LCD (tryb 4-bitowy)
LCDwc:    ldi     R20,$F0   ;E=1, RS=1 (PD4-PD5=1)
LCDwc0:   ldi     R21,$02
LCDwc1:   swap    R18       ;bajt do wysania
          mov     R19,R18
          andi    R19,$0F
          andi    R20,$F0   ;E=1, RS=pocztkowa warto (PD4-PD5=1)
          add     R20,R19
          out     PORTD,R20 ;wysanie 4-bitw do LCD (najpierw starsze, pniej modsze)
          ;(wait 4 CL)
          cbi     PORTD,$07 ;E=0, RS=pocztkowa warto (PD4-PD5=1)
          ldi     R16,$01   ;opnienie 100 us
          rcall   Wait      ;ptla opniajca
          dec     R21
          brne    LCDwc1
          ret

;Czyszczenie wywietlacza LCD i powrt kursora na pozycj 1 w 1 linii
LCDclr:   ldi     R18,$01 ;czyszczenie LCD i powrt kursora
          rcall   LCDwi   ;zapis instrukcji do LCD
          ldi     R16,$10 ;opnienie 1.6 ms
          rcall   Wait    ;ptla opniajca
          ret

;Zapis cigu ASCIIZ z pamici SRAM do wywietlacza LCD (tryb 4-bitowy)
LCDws0:   rcall   LCDwc   ;zapis znaku do LCD
LCDws:    ld      R18,Z+  ;odczyt znaku z pamici SRAM
          tst     R18
          brne    LCDws0
          ret

;Zapis cigu ASCIIZ z pamici FLASH do wywietlacza LCD (tryb 4-bitowy)
LCDwf0:   rcall   LCDwc   ;zapis znaku do LCD
LCDwf:    lpm     R18,Z+  ;odczyt znaku z pamici FLASH
          tst     R18
          brne    LCDwf0
          ret

;Ptla opniajca o zadany czas: 100u - 25.5m [s]
Wait:     mov     R2,R15  ;R15 (CLK) = czstotliwo pracy mikrokontrolera: $01-$14 (1-20 MHz)
          ldi     R17,$19
          mul     R16,R17 ;R16 (DLY) = warto opnienia: $01-$FF (100us-25.5ms)
Wait0:    movw    YL,R0
Wait1:    sbiw    YL,$01
          brne    Wait1
          dec     R2
          brne    Wait0
          ret

;Napisy dla wywietlacza LCD
Text0:    .db     "*** ADC test ***",0,"Value:",0

.dseg                ;pami danych (SRAM)
.org      $0100
sbuf:     .byte   5  ;rezerwuje N-bajtw w pamici SRAM